/* ****************************************************************************/
/*																			 PWr																	*/
/*																	 01-12-2010r															*/
/*																																						*/
/*												Program Grzegorza Koodziejczyka										*/
/*													  BEZPRZEWODOWE POCZENIE												*/
/*													 uC STM8s - RFM70 (2,4 GHz)										    */
/*																		ver. 1.0															  */
/*																  MAIN - Master     												*/
/*																																						*/
/* ****************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "stm8s_conf.h"
#include "stm8s.h"
#include "stm8s_it.h"
#include "stm8s_spi.h"
#include "RFM70.h"
#include "stm8s_tim4.h"

/*	Function prototypes ------------------------------------------------------*/
//	Initialization
void Initialization(void);
void CLK_Configuration(void);
void SPI_Configuration(void);
void init_mcu(void);
void init_port(void);

//	Delays
void power_on_delay(void);
void delay_200ms(void);
void delay_50ms(void);
void delay_5ms(void);
void delay_1ms(void);
void delay_20us(void);
void sub_program(void);

/*	Global variables----------------------------------------------------------*/
const u8 tx_buf[17]={0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,0x3c,0x3d,0x3e,0x3f,0x78};
u8 rx_buf[MAX_PACKET_LEN];

#define true 1
#define false 0

//*****************************************************************************/
//*****************************************************************************/

main()
{	
	u8 i, chksum;
	u8 temp_buf[32];
	int tempo=0;
	bool chktempo=false;
	Initialization();
	RFM70_SwitchToTxMode();
	
	while(1)
	{
		for(i = 0; i < 5; i++)delay_200ms();	// 1 sec delay
		for(i=0;i<17;i++)
		{
			temp_buf[i]=tx_buf[i];	// write data from tx_buf -> temp_buf, from where it will be sent
		}
		Send_Packet(W_TX_PAYLOAD_NOACK_CMD,temp_buf,17);
		//	-------------------------------------------------
		RFM70_SwitchToRxMode();
		for(i=0 ; i<tempo ; i++)delay_1ms();
		Receive_Packet(rx_buf);
		
		if(rx_buf[0] != 0)
		{
			chktempo=true;
			chksum = 0;
			for(i=0;i<16;i++)
			{
				chksum +=rx_buf[i]; 
			}
			
			if(chksum==rx_buf[16]&&rx_buf[0]==0x30)	// 8 bit sum of the transmited packet [16]=0x3 78, [0]=0x30
			{
				GPIO_WriteLow(GPIOD, GPIO_PIN_0);
				delay_50ms();
				delay_50ms();
				GPIO_WriteHigh(GPIOD, GPIO_PIN_0);
			}
		}
		if(chktempo==false)tempo++;
		RFM70_SwitchToTxMode();
	}
}

void Initialization()
{
  CLK_Configuration();	// Fcpu/2
 	init_mcu();	// uC init
	SPI_Configuration();	// SPI init
	enableInterrupts();
	power_on_delay();	// delay for RFM70
	RFM70_Initialize();	// RFM70 init - please look on RFM70.c
}

/*********************************************************
Function: CLK_Configuration();                                         

Description:                                                
	initialize mcu clock 
*********************************************************/
void CLK_Configuration(void)
{
	CLK_DeInit();
  CLK_HSIPrescalerConfig(CLK_PRESCALER_HSIDIV2);// Fmaster = 16/2 = 8MHz 
}

/*********************************************************
Function: init_mcu();                                         
  
Description:                                                
	initialize mcu. 
*********************************************************/
void init_mcu(void)
{
	init_port();
}
/*********************************************************
Function: SPI_Configuration;                                         

Description:                                                
	configure SPI. 
*********************************************************/
void SPI_Configuration(void)
{
	SPI_DeInit();
	SPI_Init(SPI_FIRSTBIT_MSB, SPI_BAUDRATEPRESCALER_128, SPI_MODE_MASTER, SPI_CLOCKPOLARITY_LOW, 	SPI_CLOCKPHASE_1EDGE, SPI_DATADIRECTION_2LINES_FULLDUPLEX, SPI_NSS_SOFT, 0);
	SPI_CalculateCRCCmd(DISABLE);
	SPI_Cmd(ENABLE);
}

/*********************************************************
Function: init_port();                                         
  
Description:                                                
	initialize port. 
*********************************************************/
void init_port(void)
{
	GPIO_DeInit(GPIOA);
	GPIO_DeInit(GPIOG);
	GPIO_DeInit(GPIOD);
	//	RFM70 pins definition

	GPIO_Init(GPIOG, GPIO_PIN_0, GPIO_MODE_OUT_PP_LOW_FAST);	// CSN chip select active low
	GPIO_Init(GPIOG, GPIO_PIN_1, GPIO_MODE_IN_PU_IT);	// IRQ - Maskable interrupt pin active low
	GPIO_Init(GPIOA, GPIO_PIN_5, GPIO_MODE_OUT_PP_LOW_FAST);	// CE (SPI master/slave)
	
	// 	Diodes on ports

	GPIO_Init(GPIOA, GPIO_PIN_4, GPIO_MODE_OUT_PP_LOW_FAST);	// RED		- Send
	GPIO_Init(GPIOD, GPIO_PIN_0, GPIO_MODE_OUT_PP_LOW_FAST);	// GREEN	- Receive
	
	//	Initialized values
	CE_Low();
	CSN_High();
	GPIO_WriteHigh(GPIOD, GPIO_PIN_0);
	GPIO_WriteLow(GPIOA, GPIO_PIN_4);
}

/*********************************************************
Function:  interrupt ISR_timer()                                        
                                                            
Description:                                                
 
*********************************************************/

@far @interrupt void TIM4_UPD_OVF_IRQHandler(void)	// Interrupt of TIM4 ovf
{
unsigned char i;
//;
	if(TIM4_GetFlagStatus(TIM4_IT_UPDATE) == SET)
	{
   
	TIM4_ClearFlag(TIM4_FLAG_UPDATE);
	}	
}

/*********************************************************
Function:      power_on_delay()                                    
 
Description:                                                
						Delay for RFM70
*********************************************************/
void power_on_delay(void)
{
	unsigned int i;
	for(i = 0; i<1000; i++)
	{
		delay_1ms();
	}	
}
/********************************************************

*********************************************************/
void delay_200ms(void)
{
	unsigned char j;
	for(j = 0; j<40; j++)
	{
		delay_5ms();	
	}	
}
/*********************************************************
Function: delay_50ms();                                         
                                                            
Description:                                                
	 
*********************************************************/	
void delay_50ms(void)
{
	unsigned char  j;
	for(j = 0; j<10; j++)
	{
		delay_5ms();	
	}	
}
/*********************************************************
Function: delay_5ms();                                         
                                                            
Description:                                                
	 
*********************************************************/
void delay_5ms(void)
{
	int i;
	for(i = 0; i<5; i++)   // 85*5
	{
		delay_1ms();
	}	
}	
/*********************************************************
Function: delay_1ms();                                         
                                                            
Description:                                                

*********************************************************/
void delay_1ms(void)
{
	int i;
	for(i = 0; i<400; i++)	// it isn't the right value of 1 ms delay need change
	{
		nop();
	}	
}
/*********************************************************
Function: delay_20us();                                         
                                                            
Description:                                                

*********************************************************/
void delay_20us(void)
{
	unsigned char i;
	for(i = 0; i<17; i++)
	{
		;
	}	
}

//**********************************BLACKGK********************************************/